from pathlib import Path
from ruamel.yaml import YAML
from copy import deepcopy

yaml = YAML()

class Feedstock:
    def __init__(self, feedstock_dir: Path):
        """
        met is parsed YAML from feedstock/meta.yaml
        """
        self.feedstock_dir = feedstock_dir
        with open(self.feedstock_dir / 'feedstock/meta.yaml') as f:
            self.meta = yaml.load(f)


    def _import(self, spec):
        """
        Import & return given object from recipes/ in feedstock_dir
        """
        if not hasattr(self, '_import_cache'):
            self._import_cache = {}
        module, export = spec.split(':')
        if module not in self._import_cache:
            filename = self.feedstock_dir / 'feedstock' / f'{module}.py'
            with open(filename) as f:
                ns = {}
                exec(f.read(), ns)
                self._import_cache[module] = ns

        return self._import_cache[module][export]

    def parse_recipes(self):
        """
        Parse the recipes defined in this feedstock & return them

        *Executes arbitrary code* defined in the feedstock!!!
        """
        recipes = {}
        recipes_config = self.meta['recipes']
        if isinstance(recipes_config, list):
            for r in recipes_config:
                recipes[r['id']] = self._import(r['object'])
        elif isinstance(recipes_config, dict):
            recipes = self._import(recipes_config['dict_object'])
        else:
            raise ValueError(f'Could not parse recipes config in meta.yaml')

        return recipes

    def get_expanded_meta(self):
        """
        Return full meta.yaml file, expanding recipes if needed.

        recipes are guaranteed to be a list of dictionaries with 'id' keys.
        'object' keys *may* be present, but not guaranteed.
        """
        meta_copy = deepcopy(self.meta)
        if 'recipes' in self.meta and 'dict_object' in self.meta['recipes']:
            do = self.meta['recipes']['dict_object']
            # We have a dict_object, so let's parse the recipes, and provide
            # keep just the ids, discarding the values.
            recipes = self.parse_recipes()
            meta_copy['recipes'] = [{'id': k} for k, v in recipes.items()]

        return meta_copy





import logging
from repo2docker import contentproviders
import argparse
import json
import os
import sys
import tempfile
from pathlib import Path

from pangeo_forge_runner import Feedstock

# Content providers from repo2docker are *solely* used to check out a repo
# and get their contents locally, so we can work on them.
content_providers = [
    contentproviders.Local,
    contentproviders.Zenodo,
    contentproviders.Figshare,
    contentproviders.Dataverse,
    contentproviders.Hydroshare,
    contentproviders.Swhid,
    contentproviders.Mercurial,
    contentproviders.Git,
]

logging.basicConfig(format="%(asctime)s %(msg)s", level=logging.DEBUG)
log = logging


def fetch(url, ref, checkout_path):
    """
    Fetch repo from url at ref, and check it out to checkout_path

    Uses repo2docker to detect what kinda url is going to be checked out,
    and fetches it into checkout_path. No image building or anything is
    performed.

    checkout_path should be empty.
    """
    picked_content_provider = None
    for ContentProvider in content_providers:
        cp = ContentProvider()
        spec = cp.detect(url, ref=ref)
        if spec is not None:
            picked_content_provider = cp
            log.info(
                "Picked {cp} content " "provider.\n".format(cp=cp.__class__.__name__)
            )
            break

    if picked_content_provider is None:
        raise ValueError(f'Could not fetch {url}')

    for log_line in picked_content_provider.fetch(
        spec, checkout_path, yield_output=True
    ):
        log.info(log_line, extra=dict(phase="fetching"))


def main():
    argparser = argparse.ArgumentParser()

    argparser.add_argument("--repo", help="URL to repo of feedstock to fetch")
    argparser.add_argument(
        "--ref", default=None, help="Ref to check out of the repo to build"
    )

    commands = argparser.add_subparsers(dest="command")

    expand_meta = commands.add_parser("expand-meta")
    expand_meta.add_argument(
        "--out",
        help="Path to write expanded meta.yaml out to. Will be serialized to json",
    )

    args = argparser.parse_args()

    if args.command == "expand-meta":
        with tempfile.TemporaryDirectory() as checkout_dir:
            fetch(args.repo, args.ref, checkout_dir)
            feedstock = Feedstock(Path(checkout_dir))
            expanded = feedstock.get_expanded_meta()
            with open(args.out, "w") as f:
                json.dump(expanded, f)


if __name__ == "__main__":
    main()



from setuptools import find_packages, setup

with open('README.md') as f:
    readme = f.read()

setup(
    name='pangeo-forge-runner',
    description="Commandline tool to manage pangeo-forge feedstocks",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Yuvi Panda",
    author_email="yuvipanda@gmail.com",
    version='0.1',
    packages=find_packages(),
    install_requires=[
        'jupyter-repo2docker',
        'ruamel.yaml',
        'pangeo-forge-recipes'
    ],
    entry_points={
        'console_scripts': ['pangeo-forge-runner=pangeo_forge_runner.__main__:main']
    }
)